From a0a83fb51fa1d69f4ade683fb5dd7e3db4e19130 Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Thu, 21 Apr 2022 23:03:28 +0800
Subject: [PATCH 1/9] Use closest endTime for holdAddition validity

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 13 ++++++++-----
 1 file changed, 8 insertions(+), 5 deletions(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index ab6bd78ece..2e252acdc8 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -37,6 +37,7 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             var maniaCurrent = (ManiaDifficultyHitObject)current;
             double endTime = maniaCurrent.EndTime;
             int column = maniaCurrent.BaseObject.Column;
+            double closestEndTime = 100;
 
             double holdFactor = 1.0; // Factor to all additional strains in case something else is held
             double holdAddition = 0; // Addition to the current note in case it's a hold and has to be released awkwardly
@@ -44,24 +45,26 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             // Fill up the holdEndTimes array
             for (int i = 0; i < holdEndTimes.Length; ++i)
             {
-                // If there is at least one other overlapping end or note, then we get an addition, buuuuuut...
+                // If there is at least one other overlapping end or note, then we get an addition
                 if (Precision.DefinitelyBigger(holdEndTimes[i], maniaCurrent.StartTime, 1) && Precision.DefinitelyBigger(endTime, holdEndTimes[i], 1))
                     holdAddition = 1.0;
 
-                // ... this addition only is valid if there is _no_ other note with the same ending. Releasing multiple notes at the same time is just as easy as releasing 1
-                if (Precision.AlmostEquals(endTime, holdEndTimes[i], 1))
-                    holdAddition = 0;
-
                 // We give a slight bonus to everything if something is held meanwhile
                 if (Precision.DefinitelyBigger(holdEndTimes[i], endTime, 1))
                     holdFactor = 1.25;
 
+                closestEndTime = Math.Min(closestEndTime, Math.Abs(endTime - holdEndTimes[i]));
+
                 // Decay individual strains
                 individualStrains[i] = applyDecay(individualStrains[i], current.DeltaTime, individual_decay_base);
             }
 
             holdEndTimes[column] = endTime;
 
+            // The hold addition only is valid if there is _no_ other note with the same ending. Releasing multiple notes at the same time is just as easy as releasing 1
+            if (closestEndTime < 1)
+                holdAddition = 0;
+
             // Increase individual strain in own column
             individualStrains[column] += 2.0 * holdFactor;
             individualStrain = individualStrains[column];

From e7a149af6c1c7091a4406df3fddb980b8ac5fb9f Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Thu, 21 Apr 2022 23:05:20 +0800
Subject: [PATCH 2/9] Use curve for holdAddition validity cutoff

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 3 +--
 1 file changed, 1 insertion(+), 2 deletions(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index 2e252acdc8..d5d5d5751d 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -62,8 +62,7 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             holdEndTimes[column] = endTime;
 
             // The hold addition only is valid if there is _no_ other note with the same ending. Releasing multiple notes at the same time is just as easy as releasing 1
-            if (closestEndTime < 1)
-                holdAddition = 0;
+            holdAddition *= 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
 
             // Increase individual strain in own column
             individualStrains[column] += 2.0 * holdFactor;

From 574007c07a15efd125bfa6c7076428747fe89e54 Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Fri, 22 Apr 2022 02:02:23 +0800
Subject: [PATCH 3/9] Optimise holdAddition calculation

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 4 +++-
 1 file changed, 3 insertions(+), 1 deletion(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index d5d5d5751d..04d1329eec 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -62,7 +62,9 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             holdEndTimes[column] = endTime;
 
             // The hold addition only is valid if there is _no_ other note with the same ending. Releasing multiple notes at the same time is just as easy as releasing 1
-            holdAddition *= 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
+            // Nerfs the hold addition by half if the closest release is 24ms away
+            if (holdAddition > 0)
+                holdAddition *= 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
 
             // Increase individual strain in own column
             individualStrains[column] += 2.0 * holdFactor;

From 97b4a2a105c26c4a6b56eabec85b4e0df2c424a3 Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Fri, 22 Apr 2022 12:59:00 +0800
Subject: [PATCH 4/9] Use better initial value for closestEndTime

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index 04d1329eec..f295af5f4d 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -37,7 +37,7 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             var maniaCurrent = (ManiaDifficultyHitObject)current;
             double endTime = maniaCurrent.EndTime;
             int column = maniaCurrent.BaseObject.Column;
-            double closestEndTime = 100;
+            double closestEndTime = endTime - maniaCurrent.LastObject.StartTime; // Lowest value we can assume with the current information
 
             double holdFactor = 1.0; // Factor to all additional strains in case something else is held
             double holdAddition = 0; // Addition to the current note in case it's a hold and has to be released awkwardly

From 56bbfa58e5cb08c499e6806a91725a5c186abafc Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Fri, 22 Apr 2022 13:01:41 +0800
Subject: [PATCH 5/9] Add missing absolute value of closestEndTime

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index f295af5f4d..ba964a4edd 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -37,7 +37,7 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             var maniaCurrent = (ManiaDifficultyHitObject)current;
             double endTime = maniaCurrent.EndTime;
             int column = maniaCurrent.BaseObject.Column;
-            double closestEndTime = endTime - maniaCurrent.LastObject.StartTime; // Lowest value we can assume with the current information
+            double closestEndTime = Math.Abs(endTime - maniaCurrent.LastObject.StartTime); // Lowest value we can assume with the current information
 
             double holdFactor = 1.0; // Factor to all additional strains in case something else is held
             double holdAddition = 0; // Addition to the current note in case it's a hold and has to be released awkwardly

From 8b55d3855a7112bab2e41fad4c62fc17916d5fb8 Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Fri, 22 Apr 2022 13:27:59 +0800
Subject: [PATCH 6/9] Use isOverlapping bool to determine holdAddition

---
 .../Difficulty/Skills/Strain.cs                    | 14 +++++++-------
 1 file changed, 7 insertions(+), 7 deletions(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index ba964a4edd..a5b9675836 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -41,13 +41,13 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
 
             double holdFactor = 1.0; // Factor to all additional strains in case something else is held
             double holdAddition = 0; // Addition to the current note in case it's a hold and has to be released awkwardly
+            bool isOverlapping = false;
 
             // Fill up the holdEndTimes array
             for (int i = 0; i < holdEndTimes.Length; ++i)
             {
-                // If there is at least one other overlapping end or note, then we get an addition
-                if (Precision.DefinitelyBigger(holdEndTimes[i], maniaCurrent.StartTime, 1) && Precision.DefinitelyBigger(endTime, holdEndTimes[i], 1))
-                    holdAddition = 1.0;
+                // The current note is overlapped if a previous note or end is overlapping the current note body
+                isOverlapping |= Precision.DefinitelyBigger(holdEndTimes[i], maniaCurrent.StartTime, 1) && Precision.DefinitelyBigger(endTime, holdEndTimes[i], 1);
 
                 // We give a slight bonus to everything if something is held meanwhile
                 if (Precision.DefinitelyBigger(holdEndTimes[i], endTime, 1))
@@ -61,10 +61,10 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
 
             holdEndTimes[column] = endTime;
 
-            // The hold addition only is valid if there is _no_ other note with the same ending. Releasing multiple notes at the same time is just as easy as releasing 1
-            // Nerfs the hold addition by half if the closest release is 24ms away
-            if (holdAddition > 0)
-                holdAddition *= 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
+            // The hold addition is given if there was an overlap, however it is only valid if there are no other note with a similar ending.
+            // Releasing multiple notes is just as easy as releasing 1. Nerfs the hold addition by half if the closest release is 24ms away.
+            if (isOverlapping)
+                holdAddition = 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
 
             // Increase individual strain in own column
             individualStrains[column] += 2.0 * holdFactor;

From 280cd048f662090d5bdde09191daa9ff978190dc Mon Sep 17 00:00:00 2001
From: Dean Herbert <pe@ppy.sh>
Date: Tue, 26 Apr 2022 17:30:28 +0900
Subject: [PATCH 8/9] Fix joystick settings changing enabled state of config
 level bindables

Just copied what we had in place for the mouse sensitivity setting. For
tablet settings I believe I avoided this by just hiding the settings
when not enabled. Might be a better way forward, but this is simplest
for now.

Run `TestSceneSettingsPanel` twice to get a crash without this change.
---
 .../Sections/Input/JoystickSettings.cs        | 21 ++++++++++++++++++-
 1 file changed, 20 insertions(+), 1 deletion(-)

diff --git a/osu.Game/Overlays/Settings/Sections/Input/JoystickSettings.cs b/osu.Game/Overlays/Settings/Sections/Input/JoystickSettings.cs
index c136ca6a19..60849cd6d4 100644
--- a/osu.Game/Overlays/Settings/Sections/Input/JoystickSettings.cs
+++ b/osu.Game/Overlays/Settings/Sections/Input/JoystickSettings.cs
@@ -20,6 +20,10 @@ namespace osu.Game.Overlays.Settings.Sections.Input
 
         private SettingsSlider<float> deadzoneSlider;
 
+        private Bindable<float> handlerDeadzone;
+
+        private Bindable<float> localDeadzone;
+
         public JoystickSettings(JoystickHandler joystickHandler)
         {
             this.joystickHandler = joystickHandler;
@@ -28,6 +32,10 @@ namespace osu.Game.Overlays.Settings.Sections.Input
         [BackgroundDependencyLoader]
         private void load()
         {
+            // use local bindable to avoid changing enabled state of game host's bindable.
+            handlerDeadzone = joystickHandler.DeadzoneThreshold.GetBoundCopy();
+            localDeadzone = handlerDeadzone.GetUnboundCopy();
+
             Children = new Drawable[]
             {
                 new SettingsCheckbox
@@ -40,7 +48,7 @@ namespace osu.Game.Overlays.Settings.Sections.Input
                     LabelText = JoystickSettingsStrings.DeadzoneThreshold,
                     KeyboardStep = 0.01f,
                     DisplayAsPercentage = true,
-                    Current = joystickHandler.DeadzoneThreshold,
+                    Current = localDeadzone,
                 },
             };
         }
@@ -51,6 +59,17 @@ namespace osu.Game.Overlays.Settings.Sections.Input
 
             enabled.BindTo(joystickHandler.Enabled);
             enabled.BindValueChanged(e => deadzoneSlider.Current.Disabled = !e.NewValue, true);
+
+            handlerDeadzone.BindValueChanged(val =>
+            {
+                bool disabled = localDeadzone.Disabled;
+
+                localDeadzone.Disabled = false;
+                localDeadzone.Value = val.NewValue;
+                localDeadzone.Disabled = disabled;
+            }, true);
+
+            localDeadzone.BindValueChanged(val => handlerDeadzone.Value = val.NewValue);
         }
     }
 }

From 4e260e741077cba720196c425a3e130f8bebbc79 Mon Sep 17 00:00:00 2001
From: Dean Herbert <pe@ppy.sh>
Date: Tue, 26 Apr 2022 17:32:20 +0900
Subject: [PATCH 9/9] Fix all buttons gamewide not animating properly on click

Not sure how or when this happened.
---
 osu.Game/Graphics/UserInterface/OsuButton.cs | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/osu.Game/Graphics/UserInterface/OsuButton.cs b/osu.Game/Graphics/UserInterface/OsuButton.cs
index 7c1e8d90a0..29a797bd78 100644
--- a/osu.Game/Graphics/UserInterface/OsuButton.cs
+++ b/osu.Game/Graphics/UserInterface/OsuButton.cs
@@ -108,7 +108,7 @@ namespace osu.Game.Graphics.UserInterface
             if (Enabled.Value)
             {
                 Debug.Assert(backgroundColour != null);
-                Background.FlashColour(backgroundColour.Value, 200);
+                Background.FlashColour(backgroundColour.Value.Lighten(0.4f), 200);
             }
 
             return base.OnClick(e);
From 6cca56ab77d0317a2dba6cfc114f9c3bd105d248 Mon Sep 17 00:00:00 2001
From: 63411 <62799417+molneya@users.noreply.github.com>
Date: Sat, 23 Apr 2022 10:46:12 +0800
Subject: [PATCH 7/9] Move release_threshold to const

---
 osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs | 13 +++++++++++--
 1 file changed, 11 insertions(+), 2 deletions(-)

diff --git a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
index a5b9675836..31550a8105 100644
--- a/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
+++ b/osu.Game.Rulesets.Mania/Difficulty/Skills/Strain.cs
@@ -14,6 +14,7 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
     {
         private const double individual_decay_base = 0.125;
         private const double overall_decay_base = 0.30;
+        private const double release_threshold = 24;
 
         protected override double SkillMultiplier => 1;
         protected override double StrainDecayBase => 1;
@@ -62,9 +63,17 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
             holdEndTimes[column] = endTime;
 
             // The hold addition is given if there was an overlap, however it is only valid if there are no other note with a similar ending.
             // Releasing multiple notes is just as easy as releasing 1. Nerfs the hold addition by half if the closest release is 24ms away.
             if (isOverlapping)
-                holdAddition = 1 / (1 + Math.Exp(0.5 * (24 - closestEndTime)));
+                holdAddition = 1 / (1 + Math.Exp(0.5 * (release_threshold - closestEndTime)));
 
             // Increase individual strain in own column
             individualStrains[column] += 2.0 * holdFactor;
@@ -62,9 +63,17 @@ namespace osu.Game.Rulesets.Mania.Difficulty.Skills
 
             // The hold addition is given if there was an overlap, however it is only valid if there are no other note with a similar ending.
-            // Releasing multiple notes is just as easy as releasing 1. Nerfs the hold addition by half if the closest release is 24ms away.
+            // Releasing multiple notes is just as easy as releasing 1. Nerfs the hold addition by half if the closest release is release_threshold away.
+            // holdAddition
+            //     ^
+            // 1.0 + - - - - - -+-----------
+            //     |           /
+            // 0.5 + - - - - -/   Sigmoid Curve
+            //     |         /|
+            // 0.0 +--------+-+---------------> Release Difference / ms
+            //         release_threshold
             if (isOverlapping)
